#include "displaywifiitem.h"
#include "ui_displaywifiitem.h"
#include <QDebug>
#include <QTimer>
#include <QCryptographicHash>
#include <NetworkManagerQt/Manager>
#include <NetworkManagerQt/Settings>
#include <NetworkManagerQt/WirelessSetting>
#include <NetworkManagerQt/WirelessSecuritySetting>
#include <NetworkManagerQt/Utils>
#include <QProcess>
#include <QDir>
#include <QStandardPaths>
#include <QStyleOption>
#include <QPainter>
#include "icons.h"
#include "nmconfig.h"

//
NetworkManager::ConnectionSettings::Ptr assembleWpaXPskSettings(const NetworkManager::AccessPoint::Ptr accessPoint, const QString &password, bool autoConnect)
{
    NetworkManager::ConnectionSettings::Ptr settings{new NetworkManager::ConnectionSettings{NetworkManager::ConnectionSettings::Wireless}};
    settings->setId(accessPoint->ssid());
    settings->setUuid(NetworkManager::ConnectionSettings::createNewUuid());
    settings->setAutoconnect(autoConnect);
    //Note: workaround for wrongly (randomly) initialized gateway-ping-timeout
    settings->setGatewayPingTimeout(0);

    NetworkManager::WirelessSetting::Ptr wifi_sett
        = settings->setting(NetworkManager::Setting::Wireless).dynamicCast<NetworkManager::WirelessSetting>();
    wifi_sett->setInitialized(true);
    wifi_sett->setSsid(accessPoint->ssid().toUtf8());
    wifi_sett->setSecurity("802-11-wireless-security");

    NetworkManager::WirelessSecuritySetting::Ptr security_sett
        = settings->setting(NetworkManager::Setting::WirelessSecurity).dynamicCast<NetworkManager::WirelessSecuritySetting>();
    security_sett->setInitialized(true);
    if (NetworkManager::AccessPoint::Adhoc == accessPoint->mode())
    {
        wifi_sett->setMode(NetworkManager::WirelessSetting::Adhoc);
        security_sett->setKeyMgmt(NetworkManager::WirelessSecuritySetting::WpaNone);
    } else
    {
        security_sett->setKeyMgmt(NetworkManager::WirelessSecuritySetting::WpaPsk);
    }
    security_sett->setPsk(password);
    return settings;
}

DisplayWifiItem::DisplayWifiItem(NetworkManager::WirelessDevice *device,
        NetworkManager::WirelessNetwork::Ptr network,
        QWidget *parent) :
    QWidget(parent),
    ui(new Ui::DisplayWifiItem),
    m_showType(ShowInit),
    m_active(NetworkManager::ActiveConnection::Unknown),
    m_known(false),
    m_auto(false),
    m_wirelessDevice(device),
    m_wirelessNetwork(network),
    mConnectingTimerCount(0)
{
    ui->setupUi(this);

    this->setProperty("clicked", false);
    ui->label_title->setText(network->ssid());

    //显示wifi强度图标
    this->onSignalStrengthChanged(network->signalStrength());

    connect(ui->checkBox_autoLink,&QCheckBox::toggled, this, &DisplayWifiItem::onLinkAuto);
    connect(ui->btn_link,&QToolButton::clicked, this, &DisplayWifiItem::onLinkLink);
    connect(ui->btn_forget,&QToolButton::clicked, this, &DisplayWifiItem::onLinkForget);
    connect(ui->btn_break,&QToolButton::clicked, this, &DisplayWifiItem::onLinkBreak);
    connect(ui->lineEdit_inputKey, &QLineEdit::textChanged, this, &DisplayWifiItem::onInputKeyChange);
    connect(ui->btn_cancel_inputKey,&QToolButton::clicked, this, &DisplayWifiItem::onInputKeyCancel);
    connect(ui->btn_ok_inputKey,&QToolButton::clicked, this, &DisplayWifiItem::onInputKeyOK);
    connect(ui->btn_cancel_noKey,&QToolButton::clicked, this, &DisplayWifiItem::onNoKeyCancel);
    connect(ui->btn_ok_noKey,&QToolButton::clicked, this, &DisplayWifiItem::onNoKeyOK);
    connect(ui->btn_cancel_linking,&QToolButton::clicked, this, &DisplayWifiItem::onLinkingCancel);
    connect(ui->btn_close,&QToolButton::clicked, this, &DisplayWifiItem::onLinkFinishedClose);

    connect(network.data(), &NetworkManager::WirelessNetwork::signalStrengthChanged, this, &DisplayWifiItem::onSignalStrengthChanged);
    connect(network.data(), &NetworkManager::WirelessNetwork::referenceAccessPointChanged, this, &DisplayWifiItem::onReferenceAccessPointChanged);
    connect(network.data(), &NetworkManager::WirelessNetwork::disappeared, this, &DisplayWifiItem::onDisappeared);

    m_resizeAnimation = new QPropertyAnimation(this, "itemHeight", this);
    m_resizeAnimation->setDuration(200);
    m_resizeAnimation->setEasingCurve(QEasingCurve::Linear);
    m_resizeAnimation->setStartValue(this->height());
    m_resizeAnimation->setEndValue(this->height());

    QFont font = ui->lineEdit_inputKey->font();
    font.setLetterSpacing(QFont::AbsoluteSpacing,3);
    font.setPixelSize(10);
    ui->lineEdit_inputKey->setFont(font);

    mConnectingTimer = new QTimer(this);
    mConnectingTimer->setInterval(300);
    mConnectingTimer->setSingleShot(false);
    connect(mConnectingTimer, &QTimer::timeout, [this](){
        QString dotdotdot;
        dotdotdot.fill('.', mConnectingTimerCount);
        if(mConnectingTimerCount >=3) mConnectingTimerCount = 0;
        else mConnectingTimerCount++;
        ui->label_linking->setText(tr("Connecting") + dotdotdot);
        ui->label_status->setText(tr("Connecting") + dotdotdot);

        checkActive();
    });

    showCurrentLinkStatus(ShowStatus);
}

DisplayWifiItem::~DisplayWifiItem()
{
    delete ui;
}

NetworkManager::WirelessNetwork::Ptr DisplayWifiItem::getNetwork() const
{
    return m_wirelessNetwork;
}

void DisplayWifiItem::updateActiveStatu()
{
    checkActive();

    switch (m_showType) {
    case ShowInit:
        showCurrentLinkStatus(ShowStatus);
        break;
    case ShowLink:
        showCurrentLinkStatus(ShowStatus);
        break;
    case ShowStatus:
        showCurrentLinkStatus(ShowStatus);
        break;
    case ShowInputKey:
        showCurrentLinkStatus(ShowStatus);
        break;
    case ShowLinking:
    {
        if(m_active == NetworkManager::ActiveConnection::Activating)
            showCurrentLinkStatus(ShowLinking);
        else if(m_active == NetworkManager::ActiveConnection::Activated)
            showCurrentLinkStatus(ShowLinkFinished);
        else
            showCurrentLinkStatus(ShowStatus);

        break;
    }
    case ShowLinkFinished:
        showCurrentLinkStatus(ShowStatus);
        break;
    }
}

bool DisplayWifiItem::getActiveStatu()
{
    checkActive();
    return (m_active == NetworkManager::ActiveConnection::Activated);
}

bool DisplayWifiItem::getActivingStatu()
{
    checkActive();
    return (m_active == NetworkManager::ActiveConnection::Activating);
}

void DisplayWifiItem::otherItemClicked()
{
    showCurrentLinkStatus(ShowStatus);
}

void DisplayWifiItem::onSignalStrengthChanged(int strength)
{
    if (0 >= strength)
    {
        ui->label_icon->setProperty("signal", "signal0");
    }
    else if (30 >= strength)
    {
        ui->label_icon->setProperty("signal", "signal1");
    }
    else if (70 >= strength)
    {
        ui->label_icon->setProperty("signal", "signal2");
    }
    else
    {
        ui->label_icon->setProperty("signal", "signal3");
    }

    ui->label_icon->style()->unpolish(ui->label_icon);
    ui->label_icon->style()->polish(ui->label_icon);
}

void DisplayWifiItem::onReferenceAccessPointChanged(const QString &apUni)
{
    qWarning() << "DisplayWifiItem::onReferenceAccessPointChanged()" << apUni;
//    checkActive();
//    showCurrentLinkStatus(ShowStatus);
    updateActiveStatu();
}

void DisplayWifiItem::onDisappeared(const QString &ssid)
{
    qWarning() << "DisplayWifiItem::onDisappeared()" << ssid;
}

void DisplayWifiItem::onLinkAuto()
{
    //
    bool isAuto = ui->checkBox_autoLink->isChecked();
    //已知连接, 自动连接
    NetworkManager::Connection::Ptr conn = getConnect();
    if(conn && conn->isValid())
    {
        conn->settings()->setAutoconnect(isAuto);
        conn->update(conn->settings()->toMap());
        conn->save();
        m_auto =  conn->settings()->autoconnect();
        m_known = true;
    }
}

void DisplayWifiItem::onLinkLink()
{
    showCurrentLinkStatus(ShowInputKey);
}

void DisplayWifiItem::onLinkForget()
{
    NetworkManager::Connection::Ptr conn = getConnect();
    if(conn && conn->isValid())
    {
        QString command = QString::fromUtf8("nmcli connect delete \"%1\"").arg(conn->name());
        bool res = QProcess::startDetached(command);
        if(res)
        {
            this->showCurrentLinkStatus(ShowLink);
            qWarning() <<  QString::fromUtf8("delete connect \"%1\" successed.").arg(conn->name());
        }

        //删除保存的密钥
        NMConfig::global()->removeWiFIPasswd(conn->name());
    }

    updateActiveStatu();
}

void DisplayWifiItem::onLinkBreak()
{
    for(NetworkManager::ActiveConnection::Ptr aConn: NetworkManager::activeConnections())
    {
        if(aConn->id() == m_wirelessNetwork->ssid())
        {
            QDBusPendingReply<> reply = NetworkManager::deactivateConnection(aConn->path());
            QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(reply, this);
            connect(watcher, &QDBusPendingCallWatcher::finished, [this,aConn] (QDBusPendingCallWatcher * watcher) {
                if (watcher->isError() || !watcher->isValid())
                {
                    qWarning() << QStringLiteral("断开无线连接['%1']失败: %2。").arg(aConn->id()).arg(watcher->error().message());
                }
                watcher->deleteLater();
            });
            break;
        }
    }

    updateActiveStatu();
}

void DisplayWifiItem::onInputKeyChange()
{
    QString pw = ui->lineEdit_inputKey->text();
    ui->btn_ok_inputKey->setEnabled(pw.length() >= 8);
}

void DisplayWifiItem::onInputKeyOK()
{
    NetworkManager::AccessPoint::Ptr access_point = m_wirelessNetwork->referenceAccessPoint();
    Q_ASSERT(!access_point.isNull());
    QString dev_uni = m_wirelessDevice->uni();
    QString conn_uni = access_point->uni();
    QString conn_name = access_point->ssid();

    QDBusPendingCallWatcher * watcher;
    NetworkManager::Connection::Ptr conn = getConnect();
    if(conn.isNull())
    {
        NMVariantMapMap map_settings;
        QString password = ui->lineEdit_inputKey->text();
        if(password.length() < 8) return;

        NetworkManager::ConnectionSettings::Ptr settings = assembleWpaXPskSettings(access_point, password, ui->checkBox_autoLink->isChecked());
        //保存一下密钥
        NMConfig::global()->saveWIFIPasswd(conn_name, password);
        if (settings)
        {
            map_settings = settings->toMap();
        } else {
            //
            return;
        }
        watcher = new QDBusPendingCallWatcher{NetworkManager::addAndActivateConnection(map_settings, dev_uni, conn_uni), this};
    }
    else
    {
        conn_uni = conn->path();
        QString password = ui->lineEdit_inputKey->text();
        if(password.length() < 8) return;
        //更新密码
        NetworkManager::WirelessSecuritySetting::Ptr security_sett
                = conn->settings()->setting(NetworkManager::Setting::WirelessSecurity).dynamicCast<NetworkManager::WirelessSecuritySetting>();
        security_sett->setPsk(password);
        conn->settings()->setAutoconnect(ui->checkBox_autoLink->isChecked());
        conn->update(conn->settings()->toMap());
        conn->save();
        //保存一下密钥
        NMConfig::global()->saveWIFIPasswd(conn_name, password);

        watcher = new QDBusPendingCallWatcher{NetworkManager::activateConnection(conn_uni, dev_uni, QString()), this};
    }

    //
    showCurrentLinkStatus(ShowLinking);

    QString dev_name = m_wirelessDevice->interfaceName();
    connect(watcher, &QDBusPendingCallWatcher::finished, [conn_name, dev_name, this] (QDBusPendingCallWatcher * watcher) {
        if (watcher->isError() || !watcher->isValid())
        {
            qWarning() << QStringLiteral("接口%1的连接%2激活失败：%3").arg(dev_name).arg(conn_name)
                          .arg(watcher->error().message());
            //TODO: in what form should we output the warning messages
            showCurrentLinkStatus(ShowLink);
        }
        watcher->deleteLater();
    });
}

void DisplayWifiItem::onInputKeyCancel()
{
    showCurrentLinkStatus(ShowLink);
}

void DisplayWifiItem::onNoKeyOK()
{
    //
    NetworkManager::AccessPoint::Ptr access_point = m_wirelessNetwork->referenceAccessPoint();
    Q_ASSERT(!access_point.isNull());
    QString dev_uni = m_wirelessDevice->uni();
    QString conn_uni = access_point->uni();
    QString conn_name = access_point->ssid();

    QDBusPendingCallWatcher * watcher;
    NetworkManager::Connection::Ptr conn = getConnect();
    if(conn.isNull())
    {
        NMVariantMapMap map_settings;
        NetworkManager::ConnectionSettings::Ptr settings{new NetworkManager::ConnectionSettings{NetworkManager::ConnectionSettings::Wireless}};
        settings->setId(access_point->ssid());
        settings->setUuid(NetworkManager::ConnectionSettings::createNewUuid());
        settings->setAutoconnect(ui->checkBox_autoLink->isChecked());
        map_settings = settings->toMap();
        watcher = new QDBusPendingCallWatcher{NetworkManager::addAndActivateConnection(map_settings, dev_uni, conn_uni), this};
    }
    else
    {
        conn_uni = conn->path();
        conn->settings()->setAutoconnect(ui->checkBox_autoLink->isChecked());
        conn->update(conn->settings()->toMap());
        conn->save();
        watcher = new QDBusPendingCallWatcher{NetworkManager::activateConnection(conn_uni, dev_uni, QString()), this};
    }

    //
    showCurrentLinkStatus(ShowLinking);

    QString dev_name = m_wirelessDevice->interfaceName();
    connect(watcher, &QDBusPendingCallWatcher::finished, [conn_name, dev_name, this] (QDBusPendingCallWatcher * watcher) {
        if (watcher->isError() || !watcher->isValid())
        {
            qWarning() << QStringLiteral("接口%1的连接%2激活失败：%3").arg(dev_name).arg(conn_name)
                          .arg(watcher->error().message());
            //TODO: in what form should we output the warning messages
            showCurrentLinkStatus(ShowLink);
        }
        watcher->deleteLater();
    });
}

void DisplayWifiItem::onNoKeyCancel()
{
    showCurrentLinkStatus(ShowLink);
}

void DisplayWifiItem::onLinkingCancel()
{
    showCurrentLinkStatus(ShowLink);

    //在已激活的连接中查看是否有该WIFI，有就取消连接
    for(NetworkManager::ActiveConnection::Ptr aConn: NetworkManager::activeConnections())
    {
        if(aConn->id() == m_wirelessNetwork->ssid())
        {
            QDBusPendingReply<> reply = NetworkManager::deactivateConnection(aConn->path());
            QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(reply, this);
            connect(watcher, &QDBusPendingCallWatcher::finished, [this,aConn] (QDBusPendingCallWatcher * watcher) {
                if (watcher->isError() || !watcher->isValid())
                {
                    qWarning() << QStringLiteral("取消无限连接['%1']失败: %2。").arg(aConn->id()).arg(watcher->error().message());
                }
                watcher->deleteLater();
            });
            break;
        }
    }
}

void DisplayWifiItem::onLinkFinishedClose()
{
    showCurrentLinkStatus(ShowStatus);
}

void DisplayWifiItem::paintEvent(QPaintEvent *event)
{
    QStyleOption opt;
    opt.init(this);
    QPainter p(this);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
}

void DisplayWifiItem::mousePressEvent(QMouseEvent *event)
{
//    QWidget::mousePressEvent(event);
}

void DisplayWifiItem::mouseReleaseEvent(QMouseEvent *event)
{
    if(m_showType == ShowStatus || m_showType == ShowInit) {
        emit signalItemClicked();
        if(m_active == NetworkManager::ActiveConnection::Activating) {
            showCurrentLinkStatus(ShowLinking);}
        else {
            showCurrentLinkStatus(ShowLink); }
    }
//    QWidget::mouseReleaseEvent(event);
}

int DisplayWifiItem::itemHeight()
{
    return this->height();
}

void DisplayWifiItem::setItemHeight(int height)
{
    QTimer::singleShot(10,this, [this, height](){
        this->resize(this->width(), height);
        this->setFixedHeight(height);
        emit signalItemHeightChange(height);
    });
}

void DisplayWifiItem::showCurrentLinkStatus(DisplayWifiItem::ENUM_ShowType type)
{
    switch (type) {
    case ShowInit:
        break;
    case ShowStatus:
        showStatus();
        this->setProperty("clicked", false);
        break;
    case ShowLink:
        showLink();
        this->setProperty("clicked", true);
        break;
    case ShowInputKey:
        showInputKey();
        this->setProperty("clicked", true);
        break;
    case ShowLinking:
        showLinking();
        this->setProperty("clicked", true);
        break;
    case ShowLinkFinished:
        showFinished();
        this->setProperty("clicked", true);
        break;
    }

    this->style()->unpolish(this);
    this->style()->polish(this);
}

void DisplayWifiItem::showStatus()
{
    ENUM_ShowType oldShowType = m_showType;
    int oldHeight = calcItemHeight();

    m_showType = ShowStatus;

    checkActive();
    if(m_active == NetworkManager::ActiveConnection::Activating)
    {
        ui->label_status->setText( tr("Connecting..."));
        ui->label_status->setVisible(true);
    }
    else if(m_active == NetworkManager::ActiveConnection::Activated)
    {
        ui->label_status->setText( tr("Connected"));
        ui->label_status->setVisible(true);
    }
    else
    {
        ui->label_status->setText( tr("DisConnected"));
        ui->label_status->setVisible(false);
    }

    ui->widget_link->hide();
    ui->widget_inputKey->hide();
    ui->widget_noKey->hide();
    ui->widget_linking->hide();
    ui->widget_link_finished->hide();

    int newHeight = calcItemHeight();
    if(oldShowType != ShowInit) {
        m_resizeAnimation->setStartValue(oldHeight);
        m_resizeAnimation->setEndValue(newHeight);
        if(newHeight != oldHeight) m_resizeAnimation->start();
    } else {
        this->setItemHeight(newHeight);
    }
}

void DisplayWifiItem::showLink()
{
    int oldHeight = this->height();

    m_showType = ShowLink;
    ui->label_status->hide();
    ui->widget_link->show();

    checkAuto();
    checkKnown();

    ui->checkBox_autoLink->setChecked(m_auto);
    ui->btn_forget->setVisible(m_known);
    if(m_active == NetworkManager::ActiveConnection::Activated ||
            m_active == NetworkManager::ActiveConnection::Activating)
    {
        ui->btn_link->setVisible(false);
        ui->btn_break->setVisible(true);
    }
    else
    {
        ui->btn_link->setVisible(true);
        ui->btn_break->setVisible(false);
    }

    ui->widget_inputKey->hide();
    ui->widget_noKey->hide();
    ui->widget_linking->hide();
    ui->widget_link_finished->hide();

    int newHeight = calcItemHeight();
    m_resizeAnimation->setStartValue(oldHeight);
    m_resizeAnimation->setEndValue(newHeight);
    if(newHeight != oldHeight) m_resizeAnimation->start();
}

void DisplayWifiItem::showInputKey()
{
    int oldHeight = this->height();

    m_showType = ShowInputKey;

    ui->label_status->hide();
    ui->widget_link->hide();
    ui->widget_inputKey->show();

    QString password;
    if(checkNeedPassword(password)) {
        ui->widget_inputKey->show();
        ui->widget_noKey->hide();
        if(password.length() < 8) {
            ui->lineEdit_inputKey->setText("");
            ui->btn_ok_inputKey->setEnabled(false);
        }
        else {
            ui->lineEdit_inputKey->setText(password);
            ui->btn_ok_inputKey->setEnabled(true);
        }
        ui->lineEdit_inputKey->setFocus();
    }
    else {
        ui->widget_inputKey->hide();
        ui->widget_noKey->show();
    }

    ui->label_inputKey_error->setVisible(false);
    ui->widget_linking->hide();
    ui->widget_link_finished->hide();

    int newHeight = calcItemHeight();
    m_resizeAnimation->setStartValue(oldHeight);
    m_resizeAnimation->setEndValue(newHeight);
    if(newHeight != oldHeight) m_resizeAnimation->start();
}

void DisplayWifiItem::showLinking()
{
    int oldHeight = this->height();

    m_showType = ShowLinking;

    ui->label_status->hide();
    ui->widget_link->hide();
    ui->widget_inputKey->hide();
    ui->widget_noKey->hide();
    ui->widget_linking->show();
    ui->widget_link_finished->hide();

    int newHeight = calcItemHeight();
    m_resizeAnimation->setStartValue(oldHeight);
    m_resizeAnimation->setEndValue(newHeight);
    if(newHeight != oldHeight) m_resizeAnimation->start();
}

void DisplayWifiItem::showFinished()
{
    int oldHeight = this->height();
    m_showType = ShowLinkFinished;

    ui->label_status->hide();
    ui->widget_link->hide();
    ui->widget_inputKey->hide();
    ui->widget_noKey->hide();
    ui->widget_linking->hide();
    ui->widget_link_finished->show();

    int newHeight = calcItemHeight();
    m_resizeAnimation->setStartValue(oldHeight);
    m_resizeAnimation->setEndValue(newHeight);
    if(newHeight != oldHeight) m_resizeAnimation->start();
}

int DisplayWifiItem::calcItemHeight()
{
    int height = 0;
    height += ui->main_layout->contentsMargins().top() + ui->main_layout->contentsMargins().bottom();
    height += ui->right_layout->contentsMargins().top() + ui->right_layout->contentsMargins().bottom();

    height += ui->label_title->height();

    //这里使用isHidden()而不使用isVisible()函数，若在初始化时使用isVisible()可能获取不到该控件实际的状态，有点奇怪！
    if(!ui->label_status->isHidden()) {
        height += ui->label_status->height();
        height += ui->right_layout->spacing();
    }
    if(!ui->widget_link->isHidden()) {
        height += ui->widget_link->height();
        height += ui->right_layout->spacing();
    }
    if(!ui->widget_inputKey->isHidden()) {
        height += ui->widget_inputKey->height();
        height += ui->right_layout->spacing();
    }
    if(!ui->widget_noKey->isHidden()) {
        height += ui->widget_noKey->height();
        height += ui->right_layout->spacing();
    }
    if(!ui->widget_linking->isHidden()) {
        height += ui->widget_linking->height();
        height += ui->right_layout->spacing();
    }
    if(!ui->widget_link_finished->isHidden()) {
        height += ui->widget_link_finished->height();
        height += ui->right_layout->spacing();
    }

    return height;
}

NetworkManager::ActiveConnection::State DisplayWifiItem::checkActive()
{
    m_active = NetworkManager::ActiveConnection::Unknown;
    for(NetworkManager::ActiveConnection::Ptr aConn: NetworkManager::activeConnections())
    {
        if(aConn->id() == m_wirelessNetwork->ssid())
        {
            m_active = aConn->state();
            break;
        }
    }
    if(m_active ==  NetworkManager::ActiveConnection::Activating) {
        if(!mConnectingTimer->isActive()) mConnectingTimer->start(); }
    else {
        mConnectingTimer->stop(); }

    return m_active;
}

bool DisplayWifiItem::checkKnown()
{
    m_known = false;
    NetworkManager::Connection::Ptr conn = getConnect();
    if(conn && conn->isValid())
    {
        m_known = true;
    }
    return m_known;
}

bool DisplayWifiItem::checkAuto()
{
    m_auto = true;
    NetworkManager::Connection::Ptr conn = getConnect();
    if(conn && conn->isValid())
    {
        m_auto =  conn->settings()->autoconnect();
    }
    return m_auto;
}

bool DisplayWifiItem::checkNeedPassword(QString &password)
{
    m_needPassword = false;
    NetworkManager::AccessPoint::Ptr access_point = m_wirelessNetwork->referenceAccessPoint();
    Q_ASSERT(!access_point.isNull());
    NetworkManager::WirelessSecurityType sec_type = NetworkManager::findBestWirelessSecurity(m_wirelessDevice->wirelessCapabilities()
            , true, (m_wirelessDevice->mode() == NetworkManager::WirelessDevice::Adhoc)
            , access_point->capabilities(), access_point->wpaFlags(), access_point->rsnFlags());
    switch (sec_type)
    {
    case NetworkManager::WpaPsk:
    case NetworkManager::Wpa2Psk:
    {
        m_needPassword = true;
        break;
    }
    default:
        break;
    }

    NetworkManager::Connection::Ptr conn = getConnect();
    if(conn && conn->isValid())
    {
        password = NMConfig::global()->readWIFIPasswd(conn->name());
    }
    return m_needPassword;
}

NetworkManager::Connection::Ptr DisplayWifiItem::getConnect()
{
    QString conn_name = m_wirelessNetwork->ssid();
    NetworkManager::Connection::Ptr conn;
    for (auto const & dev_conn : m_wirelessDevice->availableConnections())
    {
        if (dev_conn->settings()->id() == conn_name)
        {
            conn = dev_conn;
        }
    }
    return conn;
}

