#include "displaywireditem.h"
#include <NetworkManagerQt/Manager>
#include <NetworkManagerQt/Connection>
#include "icons.h"
#include <QDebug>
#include <QStyleOption>
#include <QPainter>
#include <QTimer>

DisplayWiredItem::DisplayWiredItem(NetworkManager::WiredDevice *device,
                                   NetworkManager::Connection::Ptr conn,
                                   QWidget *parent)
    : QWidget(parent),
      m_showType(ShowInit),
      m_active(false),
      m_wiredDevice(device),
      m_wiredConnect(conn)
{
    this->setObjectName("DisplayWiredItem");
    this->setFixedHeight(49);
    this->setMinimumWidth(276);
    this->setProperty("clicked", false);

    m_labelIcon = new QLabel(this);
    m_labelIcon->setObjectName("label_icon");
    m_labelIcon->setFixedSize(24,24);

    m_labelName = new QLabel(this);
    m_labelName->setObjectName("label_name");
    m_labelName->setText(conn->name());
    m_labelName->setWordWrap(false);
    m_labelName->setFixedHeight(29);
    m_labelName->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    QFont f = m_labelName->font();
    f.setPixelSize(14);
    m_labelName->setFont(f);

    m_labelStatus = new QLabel(this);
    m_labelStatus->setObjectName("label_status");
    m_labelStatus->setWordWrap(false);
    m_labelStatus->setFixedHeight(29);
    m_labelStatus->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    f = m_labelStatus->font();
    f.setPixelSize(14);
    m_labelStatus->setFont(f);

    m_btn = new QToolButton(this);
    m_btn->setFixedSize(84,30);
    m_btn->setText(tr("Link"));
    connect(m_btn, &QToolButton::clicked, this, &DisplayWiredItem::onBtnClicked);

    QVBoxLayout * iconLayout = new QVBoxLayout;
    iconLayout->setMargin(0);
    iconLayout->setContentsMargins(0,5,0,0);
    iconLayout->setSpacing(6);
    iconLayout->addWidget(m_labelIcon);
    iconLayout->addStretch(0);

    QVBoxLayout * textLayout = new QVBoxLayout;
    textLayout->setMargin(0);
    textLayout->setContentsMargins(0,0,0,0);
    textLayout->setSpacing(0);
    textLayout->addWidget(m_labelName);
    textLayout->addWidget(m_labelStatus);
    textLayout->addStretch(0);

    QHBoxLayout * mainLayout = new QHBoxLayout;
    mainLayout->setMargin(0);
    mainLayout->setSpacing(16);
    mainLayout->setContentsMargins(11,10,14,10);
    mainLayout->addLayout(iconLayout);
    mainLayout->addLayout(textLayout);
    mainLayout->addWidget(m_btn);

    this->setLayout(mainLayout);

    m_resizeAnimation = new QPropertyAnimation(this, "itemHeight", this);
    m_resizeAnimation->setDuration(200);
    m_resizeAnimation->setEasingCurve(QEasingCurve::Linear);
    m_resizeAnimation->setStartValue(this->height());
    m_resizeAnimation->setEndValue(this->height());

    onShowStatus();

    connect(conn.data(), &NetworkManager::Connection::updated,
            this, &DisplayWiredItem::onConnectUpdated);
    //NetworkManager::Connection::removed(const QString &path),该信号由上层来管理
    connect(conn.data(), &NetworkManager::Connection::unsavedChanged,
            this, &DisplayWiredItem::onConnectUnsavedChanged);
}

DisplayWiredItem::~DisplayWiredItem()
{
    //
}

NetworkManager::Connection::Ptr DisplayWiredItem::getConnect() const
{
    return m_wiredConnect;
}

void DisplayWiredItem::updateActiveStatu()
{
    onShowStatus();
}

bool DisplayWiredItem::getActiveStatu()
{
    return m_active;
}

int DisplayWiredItem::itemHeight()
{
    return this->height();
}

void DisplayWiredItem::setItemHeight(int height)
{
    QTimer::singleShot(10,this, [this, height](){
        this->resize(this->width(), height);
        this->setFixedHeight(height);
        emit signalItemHeightChange(height);
    });
}

void DisplayWiredItem::onBtnClicked()
{
    if(m_active)
    {
        NetworkManager::ActiveConnection::Ptr aConn;
        NetworkManager::ActiveConnection::List aConnList = NetworkManager::activeConnections();
        for(NetworkManager::ActiveConnection::Ptr p: aConnList)
        {
            if(m_wiredConnect->uuid() == p->uuid())
            {
                aConn = p;
                break;
            }
        }
        if(aConn && aConn->isValid())
        {
            QDBusPendingReply<> reply = NetworkManager::deactivateConnection(aConn->path());
            QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(reply, this);
            connect(watcher, &QDBusPendingCallWatcher::finished, [aConn] (QDBusPendingCallWatcher * watcher) {
                if (watcher->isError() || !watcher->isValid())
                {
                    qWarning() << QStringLiteral("取消有线连接['%1']失败: %2。").arg(aConn->id()).arg(watcher->error().message());
                }
                watcher->deleteLater();
            });
        }
    }
    else
    {
        QString spec_object;

        //
        QString conn_uni = m_wiredConnect->path();
        QString conn_name = m_wiredConnect->name();

        QString dev_name = m_wiredDevice->interfaceName();
        QString dev_uni = m_wiredDevice->uni();
        if(dev_uni.isEmpty())
        {
            qWarning() << QStringLiteral("找不到设备['%1']去激活有线连接['%2']。").arg(dev_name).arg(conn_name);
            return ;
        }

        QDBusPendingCallWatcher * watcher;
        watcher = new QDBusPendingCallWatcher{NetworkManager::activateConnection(conn_uni, dev_uni, spec_object), this};
        connect(watcher, &QDBusPendingCallWatcher::finished, [conn_name, dev_name] (QDBusPendingCallWatcher * watcher) {
            if (watcher->isError() || !watcher->isValid())
            {
                qWarning() << QStringLiteral("激活设备['%1']的有线连接['%2']失败: %3。").arg(dev_name).arg(conn_name).arg(watcher->error().message());
            }
            watcher->deleteLater();
        });
    }
}

void DisplayWiredItem::onConnectUpdated()
{
    //连接的状态改变
    qWarning() << "连接的状态改变:" << m_wiredConnect->name();
    QTimer::singleShot(1000,this,[this](){
        m_labelName->setText(m_wiredConnect.data()->name());
    });
//    this->updateActiveConnect();
}

void DisplayWiredItem::onConnectUnsavedChanged(bool unsaved)
{
    //
}

void DisplayWiredItem::paintEvent(QPaintEvent *event)
{
    QStyleOption opt;
    opt.init(this);
    QPainter p(this);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
}

void DisplayWiredItem::mousePressEvent(QMouseEvent *event)
{
//    QWidget::mousePressEvent(event);
}

void DisplayWiredItem::mouseReleaseEvent(QMouseEvent *event)
{
    emit signalItemClicked();
    onShowLink();
//    QWidget::mouseReleaseEvent(event);
}

void DisplayWiredItem::onShowStatus()
{
    int oldHeight = this->height();
    m_showType = ShowStatus;

    checkActive();

    m_labelStatus->setText(m_active ? tr("Connected"):tr("Disconnected"));
    m_labelStatus->setVisible(m_active);

    m_btn->setText(m_active ? tr("Break"):tr("Link"));
    m_btn->setVisible(false);

    int newHeight = m_active ? 68:49;
    m_resizeAnimation->setStartValue(oldHeight);
    m_resizeAnimation->setEndValue(newHeight);
    if(newHeight != oldHeight) m_resizeAnimation->start();

    m_labelIcon->setProperty("linked", m_active);
    m_labelIcon->style()->unpolish(m_labelIcon);
    m_labelIcon->style()->polish(m_labelIcon);
    this->setProperty("clicked", false);
    this->style()->unpolish(this);
    this->style()->polish(this);
}

void DisplayWiredItem::onShowLink()
{
    int oldHeight = this->height();
    m_showType = ShowLink;

    checkActive();

    m_labelStatus->setText(m_active ? tr("Connected"):tr("Disconnected"));
    m_labelStatus->show();

    m_btn->setText(m_active ? tr("Break"):tr("Link"));
    m_btn->setVisible(true);

    int newHeight = 68;
    m_resizeAnimation->setStartValue(oldHeight);
    m_resizeAnimation->setEndValue(newHeight);
    if(newHeight != oldHeight) m_resizeAnimation->start();

    m_labelIcon->setProperty("linked", m_active);
    m_labelIcon->style()->unpolish(m_labelIcon);
    m_labelIcon->style()->polish(m_labelIcon);
    this->setProperty("clicked", true);
    this->style()->unpolish(this);
    this->style()->polish(this);
}

bool DisplayWiredItem::checkActive()
{
    m_active = false;
    for (auto const & conn : NetworkManager::activeConnections())
    {
        if(conn->connection() == m_wiredConnect)
        {
            m_active = (conn->state() == NetworkManager::ActiveConnection::Activated);
            break;
        }
    }
    return m_active;
}
