/*COPYRIGHT_HEADER

This file is a part of nde-networkmanager.

Copyright (c)
    2015~now Palo Kisa <palo.kisa@gmail.com>

nde-networkmanager is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

COPYRIGHT_HEADER*/
#include "tray.h"

#include <QMessageBox>
#include <QApplication>
#include <QScreen>
#include <NetworkManagerQt/Manager>
#include <NetworkManagerQt/WirelessDevice>

#include "icons.h"

Tray::Tray(QObject *parent/* = nullptr*/)
    : QObject{parent}
    , mNotification{QStringLiteral("org.freedesktop.Notifications"), QStringLiteral("/org/freedesktop/Notifications"), QDBusConnection::sessionBus()}
{
    mDisplayNM = new DisplayNMWidget;
    m_timer = new QTimer(this);
    m_timer->setInterval(500);
    connect(m_timer, &QTimer::timeout, this, &Tray::onTimeout);


    mTrayIcon.setToolTip(tr("NDE Networkmanager"));
    connect(&mTrayIcon, &QSystemTrayIcon::activated, this, &Tray::onActivated);

    connect(NetworkManager::notifier(), &NetworkManager::Notifier::primaryConnectionChanged, this, &Tray::onTrayIconUpdate);
    connect(NetworkManager::notifier(), &NetworkManager::Notifier::activeConnectionAdded, this, &Tray::onTrayIconUpdate);
    connect(NetworkManager::notifier(), &NetworkManager::Notifier::activeConnectionRemoved, this, &Tray::onTrayIconUpdate);
    connect(NetworkManager::notifier(), &NetworkManager::Notifier::activeConnectionsChanged, this, &Tray::onTrayIconUpdate);
    connect(NetworkManager::notifier(), &NetworkManager::Notifier::statusChanged, this, &Tray::onTrayIconUpdate);
    connect(NetworkManager::notifier(), &NetworkManager::Notifier::activeConnectionAdded, this, &Tray::onActiveConnectAdd);
    connect(NetworkManager::notifier(), &NetworkManager::Notifier::activeConnectionRemoved, this, &Tray::onActiveConnectRemove);


    QTimer::singleShot(0, [this] { mTrayIcon.show(); });

    onTrayIconUpdate();
    mTrayIcon.show();
    mActiveConns = NetworkManager::activeConnections();
}

Tray::~Tray()
{
}

void Tray::onTrayIconUpdate()
{
    if(m_timer->isActive())
        m_timer->stop();
    m_timer->start();
}

void Tray::onActiveConnectAdd(const QString &path)
{
    NetworkManager::ActiveConnection::Ptr conn = NetworkManager::findActiveConnection(path);
    if(!conn.isNull())
    {
        QString summary, body;
        icons::Icon icon = icons::NETWORK_WIRED;
        if(conn->state() == NetworkManager::ActiveConnection::Activating){
            summary = Tray::tr("Connecting established");
            body = Tray::tr("Now connection to %1.").arg(conn->id());

            if(conn->type() == NetworkManager::ConnectionSettings::Wireless)
                icon = icons::NETWORK_WIFI_EXCELENT;
        } else {
            summary = Tray::tr("Connecting established");
            body = Tray::tr("Now connection to %1.").arg(conn->id());
            icons::Icon icon = icons::NETWORK_WIRED;
            if(conn->type() == NetworkManager::ConnectionSettings::Wireless)
                icon = icons::NETWORK_WIFI_EXCELENT;
        }



        mNotification.Notify(Tray::tr("nde-networkmanager")
                , qApp->applicationPid()
                , icons::getIconFromTheme(icon, true).name()
                , summary
                , body
                , {}
                , {}
                , -1);
    }
    mActiveConns = NetworkManager::activeConnections();
}

void Tray::onActiveConnectRemove(const QString &path)
{
    for(NetworkManager::ActiveConnection::Ptr aConn : mActiveConns)
    {
        if(aConn->path() == path)
        {
            QString summary, body;
            summary = Tray::tr("Connection lost");
            body = Tray::tr("No longer connected to %1.").arg(aConn->id());
            icons::Icon icon = icons::NETWORK_WIRED_DISCONNECTED;
            if(aConn->type() == NetworkManager::ConnectionSettings::Wireless)
            {
                icon = icons::NETWORK_WIFI_DISCONNECTED;
            }

            mNotification.Notify(Tray::tr("nde-networkmanager")
                    , qApp->applicationPid()
                    , icons::getIconFromTheme(icon, true).name()
                    , summary
                    , body
                    , {}
                    , {}
                    , -1);
            break;
        }
    }
    mActiveConns = NetworkManager::activeConnections();
}

void Tray::onActivated(const QSystemTrayIcon::ActivationReason reason)
{
    switch (reason)
    {
    case QSystemTrayIcon::Trigger:
    case QSystemTrayIcon::DoubleClick:
    {
        mDisplayNM->showDisplayNMWidget();
        //                QMenu * menu = new WindowMenu(&d->mNmModel);
        //                menu->setAttribute(Qt::WA_DeleteOnClose);
        //                menu->popup(QCursor::pos());
    }
        break;
    default:
        break;
    }
}

void Tray::onTimeout()
{
    if(m_timer->isActive())
        m_timer->stop();
    NetworkManager::ActiveConnection::Ptr prim_conn = NetworkManager::primaryConnection();
    if (prim_conn && prim_conn->isValid())
    {
        if(prim_conn->type() == NetworkManager::ConnectionSettings::Wired)
        {
            if(prim_conn->state() == NetworkManager::ActiveConnection::Activated)
            {
                mTrayIcon.setIcon(icons::getIconFromTheme(icons::NETWORK_WIRED, true));
            }
            else
            {
                mTrayIcon.setIcon(icons::getIconFromTheme(icons::NETWORK_WIRED_DISCONNECTED, true));
            }
        }
        else if(prim_conn->type() == NetworkManager::ConnectionSettings::Wireless)
        {
            //获取信号强度
            int signalStrength = 100;
            QString connName = prim_conn->connection()->name();
            for (auto const & device : NetworkManager::networkInterfaces())
            {
                if(device->type() == NetworkManager::Device::Wifi)
                {
                    NetworkManager::WirelessDevice * wirelessDevice = qobject_cast<NetworkManager::WirelessDevice *>(device.data());
                    NetworkManager::WirelessNetwork::Ptr  network = wirelessDevice->findNetwork(connName);
                    if(network)
                    {
                        signalStrength = network->signalStrength();
                        break;
                    }
                }
            }
            if(prim_conn->state() == NetworkManager::ActiveConnection::Activated)
            {
                mTrayIcon.setIcon(icons::getIconFromTheme(icons::wifiSignalIcon(signalStrength), false));
            }
            else if(prim_conn->state() == NetworkManager::ActiveConnection::Activating)
            {
                mTrayIcon.setIcon(icons::getIconFromTheme(icons::NETWORK_WIFI_ACQUIRING, false));
            }
            else
            {
                mTrayIcon.setIcon(icons::getIconFromTheme(icons::NETWORK_WIFI_DISCONNECTED, false));
            }
        }
    }
    else
    {
        mTrayIcon.setIcon(icons::getIconFromTheme(icons::NETWORK_WIRED_DISCONNECTED, true));
    }
}
